﻿using ProjectAccordionBus.Drawnings;
using ProjectAccordionBus.CollectionGenericObjects;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ProjectAccordionBus.Exceptions;

namespace ProjectAccordionBus.CollectionGenericObjects;

public class StorageCollection<T>
    where T : DrawningBus
{
    /// <summary>
    /// Словарь (хранилище) с коллекциями
    /// </summary>
    readonly Dictionary<string, ICollectionGenericObjects<T>> _storages;
    
    /// <summary>
    /// Возвращение списка названий коллекций
    /// </summary>
    public List<string> Keys => _storages.Keys.ToList();

    /// <summary>
    /// Ключевое слово, с которого должен начинаться файл
    /// </summary>
    private readonly string _collectionKey = "CollectionsStorage";
    /// <summary>
    /// Разделитель для записи ключа и значения элемента словаря
    /// </summary>
    private readonly string _separatorForKeyValue = "|";
    /// <summary>
    /// Разделитель для записей коллекции данных в файл
    /// </summary>
    private readonly string _separatorItems = ";";


    /// <summary>
    /// Конструктор
    /// </summary>
    public StorageCollection()
    {
        _storages = new Dictionary<string, ICollectionGenericObjects<T>>();
    }
    
    /// <summary>
    /// Добавление коллекции в хранилище
    /// </summary>
    /// <param name="name">Название коллекции</param>
    /// <param name="collectionType">тип коллекции</param>
    public void AddCollection(string name, CollectionType collectionType)
    {
        if (name == null || _storages.ContainsKey(name))
            return;
        switch (collectionType)
        {
            case CollectionType.None:
                return;
            case CollectionType.Massive:
                _storages[name] = new MassiveGenericObjects<T>();
                return;
            case CollectionType.List:
                _storages[name] = new ListGenericObjects<T>();
                return;
            default: break;
        }
    }
    /// <summary>
    /// Удаление коллекции
    /// </summary>
    /// <param name="name">Название коллекции</param>
    public void DelCollection(string name)
    {
        if (_storages.ContainsKey(name))
            _storages.Remove(name);
    }
    
    /// <summary>
    /// Доступ к коллекции
    /// </summary>
    /// <param name="name">Название коллекции</param>
    /// <returns></returns>
    public ICollectionGenericObjects<T>? this[string name]
    {
        get
        {
            if (name == "")
            {
                return null;
            }
            return _storages[name];
        }
    }

    public void SaveData(string filename)
    {
        if (_storages.Count == 0)
        {
            throw new ArgumentException("В хранилище отсутствуют коллекции для сохранения");
        }
        if (File.Exists(filename))
        {
            File.Delete(filename);
        }
        using (StreamWriter writer = new(filename))
        {
            writer.Write(_collectionKey);
            foreach (KeyValuePair<string, ICollectionGenericObjects<T>> value in _storages)
            {
                writer.Write(Environment.NewLine);
                // не сохраняем пустые коллекции
                if (value.Value.Count == 0)
                {
                    continue;
                }
                writer.Write(value.Key);
                writer.Write(_separatorForKeyValue);
                writer.Write(value.Value.GetCollectionType);
                writer.Write(_separatorForKeyValue);
                writer.Write(value.Value.MaxCount);
                writer.Write(_separatorForKeyValue);

                foreach (T? item in value.Value.GetItems())
                {
                    string data = item?.GetDataForSave() ?? string.Empty;
                    if (string.IsNullOrEmpty(data))
                    {
                        continue;
                    }
                    writer.Write(data);
                    writer.Write(_separatorItems);
                }
            }
        }
    }

    public void LoadData(string filename)
    {
        if (!File.Exists(filename))
        {
            throw new FileNotFoundException($"{filename} не существует");
        }
        using (StreamReader reader = new(filename))
        {
            string line = reader.ReadLine();
            if (line == null || line.Length == 0)
            {
                throw new IOException("Файл не подходит");
            }
            if (!line.Equals(_collectionKey))
            {

                throw new IOException("В файле неверные данные");
            }
            _storages.Clear();
            while ((line = reader.ReadLine()) != null)
            {
                string[] record = line.Split(_separatorForKeyValue,
                StringSplitOptions.RemoveEmptyEntries);
                if (record.Length != 4)
                {
                    continue;
                }
                CollectionType collectionType = (CollectionType)Enum.Parse(typeof(CollectionType), record[1]);
                ICollectionGenericObjects<T>? collection = StorageCollection<T>.CreateCollection(collectionType);
                if (collection == null)
                {
                    throw new Exception("Не удалось создать коллекцию");
                }
                collection.MaxCount = Convert.ToInt32(record[2]);
                string[] set = record[3].Split(_separatorItems,
                StringSplitOptions.RemoveEmptyEntries);
                foreach (string elem in set)
                {
                    if (elem?.CreateDrawningBus() is T armoredCar)
                    {
                        try
                        {
                            if (collection.Insert(armoredCar) == -1)
                            {
                                throw new Exception("Объект не удалось добавить в коллекцию: " + record[3]);
                            }
                        }
                        catch (CollectionOverflowException ex)
                        {
                            throw new Exception("Коллекция переполнена", ex);
                        }
                    }
                }
                _storages.Add(record[0], collection);
            }
        }
    }

    /// <summary>
    /// Создание коллекции по типу
    /// </summary>
    /// <param name="collectionType"></param>
    /// <returns></returns>
    private static ICollectionGenericObjects<T>?
        CreateCollection(CollectionType collectionType)
        {
            return collectionType switch
            {
                CollectionType.Massive => new MassiveGenericObjects<T>(),
                CollectionType.List => new ListGenericObjects<T>(),
                _ => null,
            };
        }
}
