﻿using ProjectAccordionBus.Entities;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectAccordionBus.Drawnings;
public class DrawningBus
{
    /// <summary>
    /// Класс-сущность
    /// </summary>
    public EntityBus? EntityBus { get; protected set; }

    /// <summary>
    /// Ширина окна
    /// </summary>
    private int? _pictureWidth;

    /// <summary>
    /// Высота окна
    /// </summary>
    private int? _pictureHeight;

    /// <summary>
    /// Левая координата прорисовки транспорта
    /// </summary>
    protected int? _startPosX;

    /// <summary>
    /// Верхняя координата прорисовки транспорта
    /// </summary>
    protected int? _startPosY;

    /// <summary>
    /// Ширина прорисовки транспорта
    /// </summary>
    private readonly int _drawningBusWidth = 50;

    /// <summary>
    /// Высота прорисовки транспорта
    /// </summary>
    private readonly int _drawningBusHeight = 35;

    /// <summary>
    /// Координата X объекта
    /// </summary>
    public int? GetPosX => _startPosX;

    /// <summary>
    /// Координата Y объекта
    /// </summary>
    public int? GetPosY => _startPosY;

    /// <summary>
    /// Ширина объекта
    /// </summary>
    public int GetWidth => _drawningBusWidth;

    /// <summary>
    /// Высота объекта
    /// </summary>
    public int GetHight => _drawningBusHeight;

    // Пустой конструктор
    public DrawningBus()
    {
        _pictureWidth = null;
        _pictureHeight = null;
        _startPosX = null;
        _startPosY = null;
    }

    // Конструктор
    public DrawningBus(int speed, double weight, Color bodyColor) : this()
    {
        EntityBus = new EntityBus(speed, weight, bodyColor);
    }

    // Конструктор
    protected DrawningBus(int drawningBusWidth, int drawningBusHeight) : this()
    {
        _drawningBusWidth = drawningBusWidth;
        _drawningBusHeight = drawningBusHeight;
    }

    public DrawningBus(EntityBus? entityBus)
    {
        if (entityBus != null)
        {
            EntityBus = entityBus;
        }
    }

    public bool SetPictureSize(int width, int height)
    {
        if (_drawningBusWidth <= width && _drawningBusHeight <= height)
        {
            _pictureWidth = width;
            _pictureHeight = height;
            if (_startPosX.HasValue && _startPosY.HasValue)
            {
                if (_startPosX + _drawningBusWidth > width)
                {
                    _startPosX = width - _drawningBusWidth;
                }
                if (_startPosY + _drawningBusHeight > height)
                {
                    _startPosY = height - _drawningBusHeight;
                }
            }
            return true;
        }
        return false;
    }

    public void SetPosition(int x, int y)
    {
        if (!_pictureHeight.HasValue || !_pictureWidth.HasValue)
        {
            return;
        }
        _startPosX = x;
        _startPosY = y;
        if (x < 0)
        {
            _startPosX = 0;
        }

        if (y < 0)
        {
            _startPosY = 0;
        }

        if (x + _drawningBusWidth > _pictureWidth)
        {
            _startPosX = _pictureWidth - _drawningBusWidth;
        }

        if (y + _drawningBusHeight > _pictureHeight)
        {
            _startPosY = _pictureHeight - _drawningBusHeight;
        }
    }

    public bool MoveTransport(DirectionType direction)
    {
        if (EntityBus == null || !_startPosX.HasValue || !_startPosY.HasValue)
        {
            return false;
        }
        switch (direction)
        {
            //влево
            case DirectionType.Left:
                if (_startPosX.Value - EntityBus.Step > 0)
                {
                    _startPosX -= (int)EntityBus.Step;
                }
                return true;
            //вверх
            case DirectionType.Up:
                if (_startPosY.Value - EntityBus.Step > 0)
                {
                    _startPosY -= (int)EntityBus.Step;
                }
                return true;
            // вправо
            case DirectionType.Right:
                if (_startPosX.Value + _drawningBusWidth + EntityBus.Step < _pictureWidth.Value)
                {
                    _startPosX += (int)EntityBus.Step;
                }
                return true;
            //вниз
            case DirectionType.Down:
                if (_startPosY.Value + _drawningBusHeight + EntityBus.Step < _pictureHeight.Value)
                {
                    _startPosY += (int)EntityBus.Step;
                }
                return true;
            default:
                return false;
        }
    }

    public virtual void DrawTransport(Graphics g)
    {
        if (EntityBus == null || !_startPosX.HasValue || !_startPosY.HasValue)
        {
            return;
        }

        Pen pen = new(Color.Black);
        Brush bodyBrush = new SolidBrush(EntityBus.BodyColor);

        // границы автобуса
        g.FillRectangle(bodyBrush, _startPosX.Value, _startPosY.Value, 50, 30);
        g.DrawRectangle(pen, _startPosX.Value, _startPosY.Value, 50, 30);

        // окна
        Brush brushBlue = new SolidBrush(Color.Blue);
        g.FillEllipse(brushBlue, _startPosX.Value + 5, _startPosY.Value + 5, 5, 10);
        g.FillEllipse(brushBlue, _startPosX.Value + 40, _startPosY.Value + 5, 5, 10);
        g.FillEllipse(brushBlue, _startPosX.Value + 30, _startPosY.Value + 5, 5, 10);

        g.DrawEllipse(pen, _startPosX.Value + 5, _startPosY.Value + 5, 5, 10);
        g.DrawEllipse(pen, _startPosX.Value + 40, _startPosY.Value + 5, 5, 10);
        g.DrawEllipse(pen, _startPosX.Value + 30, _startPosY.Value + 5, 5, 10);

        // двери
        g.FillRectangle(bodyBrush, _startPosX.Value + 18, _startPosY.Value + 10, 10, 20);

        g.DrawRectangle(pen, _startPosX.Value + 18, _startPosY.Value + 10, 10, 20);

        // колеса
        g.FillEllipse(bodyBrush, _startPosX.Value + 5, _startPosY.Value + 25, 10, 10);
        g.FillEllipse(bodyBrush, _startPosX.Value + 35, _startPosY.Value + 25, 10, 10);

        g.DrawEllipse(pen, _startPosX.Value + 5, _startPosY.Value + 25, 10, 10);
        g.DrawEllipse(pen, _startPosX.Value + 35, _startPosY.Value + 25, 10, 10);
    }
}

